/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.addressing;

import org.apache.ws.util.NameUtils;
import javax.xml.namespace.QName;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import java.util.Iterator;

/**
 * A set of WS-Addressing-related utility methods.
 */
public abstract class AddressingUtils
{
   /**
    * An array that contains the namespace URIs (as Strings) for the various versions of WSA that we support. The URIs
    * are indexed in the array in order of preference.
    */
   private static final String[] SUPPORTED_WSA_NSURIS =
      new String[]
      {
         org.apache.ws.addressing.v2004_08.AddressingConstants.NSURI_ADDRESSING_SCHEMA,
         org.apache.ws.addressing.v2004_03.AddressingConstants.NSURI_ADDRESSING_SCHEMA,
         org.apache.ws.addressing.v2003_03.AddressingConstants.NSURI_ADDRESSING_SCHEMA
      };

   /**
    * Returns the WS-Addressing namespace URI for the specified SOAP header, or null if the header is <code>null</code>
    * or does not contain any WS-Addressing header elements.
    *
    * @param soapHeader a SOAP header
    *
    * @return the WS-Addressing namespace URI for the specified SOAP header
    */
   public static String getAddressingNamespaceURI( SOAPHeader soapHeader )
   {
      String wsaNsURI = null;
      if ( soapHeader != null )
      {
         Iterator headerElemIter = soapHeader.examineAllHeaderElements(  );
         while ( headerElemIter.hasNext(  ) )
         {
            SOAPHeaderElement headerElem     = (SOAPHeaderElement) headerElemIter.next(  );
            Name              headerElemName = headerElem.getElementName(  );

            // NOTE: Action is the only message info header that is required by all versions of WS-A
            if ( "Action".equals( headerElemName.getLocalName(  ) ) )
            {
               for ( int i = 0; i < SUPPORTED_WSA_NSURIS.length; i++ )
               {
                  if ( SUPPORTED_WSA_NSURIS[i].equals( headerElemName.getURI(  ) ) )
                  {
                     wsaNsURI = SUPPORTED_WSA_NSURIS[i];
                     break;
                  }
               }
            }
         }
      }

      return wsaNsURI;
   }

   /**
    * @param soapHeader
    * @param headerElemName
    *
    * @return the header element with the name that was specified
    */
   public static SOAPHeaderElement getHeaderElement( SOAPHeader soapHeader,
                                                     QName      headerElemName )
   {
      if ( headerElemName == null )
      {
         throw new IllegalArgumentException( "Header element name must not be null." );
      }

      SOAPHeaderElement headerElem = null;
      if ( soapHeader != null )
      {
         Iterator headerElemIter = soapHeader.examineAllHeaderElements(  );
         while ( headerElemIter.hasNext(  ) )
         {
            SOAPHeaderElement elem = (SOAPHeaderElement) headerElemIter.next(  );
            if ( NameUtils.toQName( elem.getElementName(  ) ).equals( headerElemName ) )
            {
               headerElem = elem;
               break;
            }
         }
      }

      return headerElem;
   }
}