/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.addressing.handler;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.Soap1_1Constants;
import org.apache.ws.addressing.AddressingUtils;
import org.apache.ws.resource.faults.FaultException;
import org.apache.ws.resource.i18n.Keys;
import org.apache.ws.resource.i18n.MessagesImpl;
import org.apache.ws.util.NameUtils;
import org.apache.ws.util.i18n.Messages;
import javax.xml.namespace.QName;
import javax.xml.rpc.handler.GenericHandler;
import javax.xml.rpc.handler.MessageContext;
import javax.xml.rpc.handler.soap.SOAPMessageContext;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import java.net.URI;
import java.net.URISyntaxException;

/**
 * A JAX-RPC {@link javax.xml.rpc.handler.Handler} that provides minimal support for WS-Addressing.
 *
 * @author Sal Campana
 */
public class WSAddressingHandler
   extends GenericHandler
{
   private static final Log      LOG                                   =
      LogFactory.getLog( WSAddressingHandler.class.getName(  ) );
   private static final Messages MSG                                   = MessagesImpl.getInstance(  );

   /** DOCUMENT_ME */
   public static final String CONTEXT_PROP_WSA_NAMESPACE_URI        = "wsa.namespace.uri";

   /** DOCUMENT_ME */
   public static final String CONTEXT_PROP_WSA_RESPONSE_ACTION      = "wsa.response.action";

   /** DOCUMENT_ME */
   public static final String CONTEXT_PROP_WSA_RESPONSE_DESTINATION = "wsa.response.destination";

   /** DOCUMENT_ME */
   public static final String CONTEXT_PROP_WSA_REQUEST_DESTINATION = "wsa.request.destination";

   /** DOCUMENT_ME */
   public static final String CONTEXT_PROP_WSA_REQUEST_ACTION = "wsa.request.action";

   /**
    * DOCUMENT_ME
    *
    * @param msgContext DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String getAddressingAction( SOAPMessageContext msgContext )
   {
      SOAPHeaderElement actionHeaderElem =
         AddressingUtils.getHeaderElement( getSOAPHeader( msgContext ),
                                           new QName( getAddressingNamespaceURI( msgContext ),
                                                      "Action" ) );
      if ( actionHeaderElem == null )
      {
         LOG.debug( MSG.getMessage( Keys.NO_WSA_ACTION ) );
         throw new FaultException( Soap1_1Constants.FAULT_CLIENT,
                                   "A WS-Addressing Action SOAP header element is required by this endpoint." );
      }

      String action = actionHeaderElem.getValue(  );
      try
      {
         new URI( action );
      }
      catch ( URISyntaxException urise )
      {
         LOG.debug( MSG.getMessage( Keys.INVALID_WSA_ACTION, action ) );
         throw new FaultException( Soap1_1Constants.FAULT_CLIENT,
                                   "The WS-Addressing Action specified in the SOAP header is not a valid URI." );
      }

      LOG.debug( MSG.getMessage( Keys.FOUND_WSA_ACTION, action ) );
      return action;
   }

   /**
    * DOCUMENT_ME
    *
    * @param msgContext DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String getAddressingNamespaceURI( SOAPMessageContext msgContext )
   {
      String wsaNsURI = (String) msgContext.getProperty( CONTEXT_PROP_WSA_NAMESPACE_URI );
      if ( wsaNsURI == null )
      {
         wsaNsURI = AddressingUtils.getAddressingNamespaceURI( getSOAPHeader( msgContext ) );
         msgContext.setProperty( CONTEXT_PROP_WSA_NAMESPACE_URI, wsaNsURI );
      }

      return wsaNsURI;
   }

   /**
    * DOCUMENT_ME
    *
    * @param msgContext DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static String getAddressingTo( SOAPMessageContext msgContext )
   {
      SOAPHeaderElement toHeaderElem =
         AddressingUtils.getHeaderElement( getSOAPHeader( msgContext ),
                                           new QName( getAddressingNamespaceURI( msgContext ),
                                                      "To" ) );
      String            addressingTo;
      if ( toHeaderElem == null )
      {
         LOG.debug( MSG.getMessage( Keys.NO_WSA_TO ) );
         throw new FaultException( Soap1_1Constants.FAULT_CLIENT,
                                   "A WS-Addressing To SOAP header element is required by this endpoint." );
      }

      addressingTo = toHeaderElem.getValue(  );

      return addressingTo;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName[] getHeaders(  )
   {
      return new QName[0];
   }

   /**
    * DOCUMENT_ME
    *
    * @param msgContext DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static SOAPHeader getSOAPHeader( SOAPMessageContext msgContext )
   {
      try
      {
         return msgContext.getMessage(  ).getSOAPHeader(  );
      }
      catch ( SOAPException soape )
      {
         throw new RuntimeException( "Failed to extract header from SOAP message.", soape );
      }
   }

   /**
    * Checks for the required WS-Addressing Headers and adds them to the MessageContext.
    *
    * @param messageContext
    * @return always true
    */
   public boolean handleRequest( MessageContext messageContext )
   {
      SOAPMessageContext soapMsgContext = (SOAPMessageContext) messageContext;

      String             addressingAction       = getAddressingAction( soapMsgContext );
      String             addressingTo           = getAddressingTo( soapMsgContext );
      String             addressingNamespaceURI = getAddressingNamespaceURI( soapMsgContext );

      soapMsgContext.setProperty( CONTEXT_PROP_WSA_NAMESPACE_URI, addressingNamespaceURI );
      soapMsgContext.setProperty( CONTEXT_PROP_WSA_REQUEST_ACTION, addressingAction );
      soapMsgContext.setProperty( CONTEXT_PROP_WSA_REQUEST_DESTINATION, addressingTo );
      return true;
   }

   /**
    * @param messageContext
    * @return always true
    */
   public boolean handleResponse( MessageContext messageContext )
   {
      LOG.debug( MSG.getMessage( Keys.HANDLING_RESPONSE ) );

      try
      {
         SOAPHeader header = ( (SOAPMessageContext) messageContext ).getMessage(  ).getSOAPHeader(  );

         //wsa:Action
         String wsaAction = (String) messageContext.getProperty( CONTEXT_PROP_WSA_RESPONSE_ACTION ); //how would this ever be null?
         if ( wsaAction == null )
         {
            wsaAction = (String) messageContext.getProperty( CONTEXT_PROP_WSA_NAMESPACE_URI ) + "/anonymous";
         }

         SOAPHeaderElement actionHeaderElem =
            header.addHeaderElement( NameUtils.createName( "Action", "wsa",
                                                           (String) messageContext.getProperty( CONTEXT_PROP_WSA_NAMESPACE_URI ) ) );
         actionHeaderElem.setValue( wsaAction );

         //wsa:To
         String wsaTo = (String) messageContext.getProperty( CONTEXT_PROP_WSA_RESPONSE_DESTINATION );
         if ( wsaTo == null )
         {
            wsaTo = (String) messageContext.getProperty( CONTEXT_PROP_WSA_NAMESPACE_URI ) + "/anonymous";
         }

         SOAPHeaderElement toHeaderElem =
            header.addHeaderElement( NameUtils.createName( "To", "wsa",
                                                           (String) messageContext.getProperty( CONTEXT_PROP_WSA_NAMESPACE_URI ) ) );
         toHeaderElem.setValue( wsaTo );
      }
      catch ( SOAPException e )
      {
         LOG.error( MSG.getMessage( Keys.INTERNAL_SERVER_ERROR ),
                    e );
         throw new FaultException( Soap1_1Constants.FAULT_SERVER,
                                   "An exception occurred while adding the required WS-Addressing fields to the response." );
      }

      return true;
   }
}