/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.metadata;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Represents the WS-MetadataExchange configuration information for a
 * particular type of WS-Resource.
 *
 * @author Sal Campana
 */
public class MetadataConfiguration
{
   /*
    * Internal Map for storing metatdata.
    *
    * The map will have the structure of:
    *
    * key=dialect value=Map (dialect-specific Map)
    *
    * The dialect-specific Map will have the structure:
    *
    * Two Entries: key= IDENTIFIER_MAP value= A Map keyed on Identifier
    * keyed-to the metatdata
    *
    * key= COMPLETE_METADATA_LIST value= A List containing ALL metadata given
    * this dialect (this includes all entries from IDENTIFIER_MAP)
    *
    */
   private Map m_dialectToIdentifierMapMap = new HashMap(  );

   /* list of all the MetadataEntry's */
   private List m_metadataEntryList;

   /**
    * Creates a new <code>MetadataConfigImpl</code> object.
    */
   public MetadataConfiguration(  )
   {
      this( new ArrayList(  ) );
   }

   /**
    * Creates a new <code>MetadataConfigImpl</code> object.
    *
    * @param metadataEntryList a list of {@link MetadataEntry}s
    */
   public MetadataConfiguration( List metadataEntryList )
   {
      m_metadataEntryList = metadataEntryList;
      initMetadataEntryMap(  );
   }

   /**
    * Returns all known metadata for all dialects.
    *
    * @return A list of all metadata
    */
   public List getAllMetadata(  )
   {
      return m_metadataEntryList;
   }

   /**
    * Returns the metadata given a dialect and an identifier.
    *
    * @param dialect
    * @param identifierUri
    * @return A Map keyed on dialect keyed-to a List of metadata
    */
   public Map getMetadata( String dialect,
                           String identifierUri )
   {
      Map results = new HashMap(  );
      if ( ( dialect != null ) && !"".equals( dialect ) ) // only get metatdata// for specific dialect
      {
         Map dialectMap = (Map) m_dialectToIdentifierMapMap.get( dialect );
         if ( dialectMap != null )
         {
            if ( identifierUri != null )
            {
               Object metatdata = dialectMap.get( identifierUri );
               if ( metatdata != null )
               {
                  List list = new ArrayList(  );
                  list.add( metatdata );
                  results.put( dialect, list );
               }
            }
            else
            {
               // based on dialect get all entries
               results.put( dialect,
                            new ArrayList( dialectMap.values(  ) ) );
            }
         }
      }

      // if dialect was not defined then they are saying "send all!"
      else
      {
         return buildDialectToListMap(  );
      }

      return results;
   }

   /**
    * Returns metadata based on dialect
    *
    * @param dialect
    * @return A Map keyed on dialect keyed-to a List of metadata
    */
   public Map getMetadata( String dialect )
   {
      return getMetadata( dialect, null );
   }

   /**
    * DOCUMENT_ME
    *
    * @param metadataEntry DOCUMENT_ME
    */
   public void addMetadataEntry( MetadataEntry metadataEntry )
   {
      m_metadataEntryList.add( metadataEntry );
      addToMetadataMap( metadataEntry );
   }

   /**
    * Adds the specified metadata entry to the undelying lookup structures.
    *
    * precondition: The structures have already been setup for the metadata
    *
    * @param metadataEntry
    */
   protected void addToMetadataMap( MetadataEntry metadataEntry )
   {
      String dialect                      = metadataEntry.getDialect(  );
      String identifier                   = metadataEntry.getIdentifier(  );
      Map    identifierToMetadataEntryMap = (Map) m_dialectToIdentifierMapMap.get( dialect );
      if ( identifierToMetadataEntryMap == null )
      {
         // build the entries for the maps
         identifierToMetadataEntryMap = new HashMap(  );
         m_dialectToIdentifierMapMap.put( dialect, identifierToMetadataEntryMap );
      }

      // if there is an identifier then the metadata is registered with it
      // else it only lives in the list of all metadata
      if ( ( identifier != null ) && !"".equals( identifier ) )
      {
         identifierToMetadataEntryMap.put( identifier, metadataEntry );
      }
   }

   private Map buildDialectToListMap(  )
   {
      Map      dialectToMetadataList = new HashMap(  );

      Iterator iter = m_dialectToIdentifierMapMap.keySet(  ).iterator(  );
      while ( iter.hasNext(  ) )
      {
         String dialect       = (String) iter.next(  );
         Map    identifierMap = (Map) m_dialectToIdentifierMapMap.get( dialect );
         dialectToMetadataList.put( dialect,
                                    new ArrayList( identifierMap.values(  ) ) );
      }

      return dialectToMetadataList;
   }

   /**
    * Loads up lookup structures with the metadata.
    *
    * This method initializes a Map for a given dialect if it doesn't already
    * exist. The Map is keyed using 2 constants for pulling a map of
    * identifiers->metatada AND a List of all metatdata.
    */
   private synchronized void initMetadataEntryMap(  )
   {
      for ( int i = 0; i < m_metadataEntryList.size(  ); i++ )
      {
         MetadataEntry metadataEntry = (MetadataEntry) m_metadataEntryList.get( i );
         addToMetadataMap( metadataEntry );
      }
   }
}