/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource;

import javax.xml.soap.SOAPMessage;
import java.net.URI;
import java.net.URL;
import java.util.Iterator;

/**
 * Context information pertaining to a specific request being processed by the WSRF
 * resource invocation framework. Provides methods for obtaining the {@link ResourceHome}
 * and {@link Resource} objects associated with the request. Also provides access to
 * some service related information such as the service endpoint URL.
 */
public interface ResourceContext
{
   /**
    * @see javax.xml.rpc.handler.MessageContext#getProperty(String)
    */
   public abstract Object getProperty( String name );

   /**
    * @see javax.xml.rpc.handler.MessageContext#getPropertyNames()
    */
   public abstract Iterator getPropertyNames(  );

   /**
    * @see javax.xml.rpc.handler.MessageContext#setProperty(String, Object)
    */
   void setProperty( String name,
                     Object value );

   /**
    * Returns the value of the request's wsa:Action header element.
    *
    * @return the value of the request's wsa:Action header element
    */
   String getRequestAction(  );

   /**
    * Returns the resource instance that was targeted by the request associated
    * with this context.
    *
    * @return the resource instance that was targeted by the request associated
    *         with this context
    *
    * @throws ResourceException upon failure to lookup a resource instance from the home
    */
   Resource getResource(  )
   throws ResourceException;

   /**
    * Gets <code>ResourceHome</code> associated with the service.
    *
    * @return <code>ResourceHome</code> associated with the service. Cannot be null.
    */
   ResourceHome getResourceHome(  );

   /**
    * Sets the value that should be used for the response's wsa:Action header element.
    *
    * @param action the value that should be used for the response's wsa:Action header element;
    *        if null, a default value will be generated by the resource invocation framework
    */
   void setResponseAction( URI action );

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   SOAPMessage getSOAPMessage(  );

   /**
    * Gets name of the service.
    *
    * @return The name of the service.
    */
   String getServiceName(  );

   /**
    * Gets the endpoint URL of the service.
    *
    * @return The endpoint URL of the service
    */
   URL getServiceURL(  );

   /**
    * @see javax.xml.rpc.handler.MessageContext#containsProperty(String)
    */
   boolean containsProperty( String name );

   /**
    * @see javax.xml.rpc.handler.MessageContext#removeProperty(String)
    */
   void removeProperty( String name );
}