/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.handler;

import javax.xml.namespace.QName;

/**
 * LOG-DONE
 * Abstract class to provide default underlying behaviour for
 * the SOAPMethodNameMap.
 *
 * @author Sal Campana
 */
public abstract class AbstractSoapMethodNameMap
   implements SoapMethodNameMap
{
   /**
    * The Parent SoapMethodNameMap.
    */
   SoapMethodNameMap m_parent;

   /**
    * Returns a default name for a method based on a request's QName.
    *
    * @param requestName QName of the incoming request.
    *
    * @return a default name for the method.
    */
   public abstract String getDefaultMethodName( QName requestName );

   /**
    * Returns the method name for a given incoming request QName.
    *
    * @param requestName
    *
    * @return Method name to invoke.
    */
   public final String getMethodName( QName requestName )
   {
      SoapMethodNameMap parent     = getParent(  );
      String            methodName = null;
      if ( parent != null )
      {
         methodName = parent.getMethodName( requestName );
      }

      if ( methodName == null )
      {
         methodName = getDefaultMethodName( requestName );
      }

      return methodName;
   }

   /**
    * Returns the method name for an operation which
    * is registered via its WS-Addressing Action header
    * value.
    *
    * @param action The WS-AAddressing Action Header value.
    *
    * @return the name of the operation OR null if there is no Action registered.
    */
   public final String getMethodNameFromAction( String action )
   {
      if ( action == null )
      {
         return null;
      }

      SoapMethodNameMap parent     = getParent(  );
      String            methodName = null;
      if ( parent != null )
      {
         methodName = parent.getMethodNameFromAction( action );
      }

      if ( methodName == null )
      {
         methodName = lookupMethodNameFromAction( action );
      }

      return methodName;
   }

   /**
    * Sets the parent SoapMethodNameMap to be searched.
    *
    * @param parent
    */
   public void setParent( SoapMethodNameMap parent )
   {
      m_parent = parent;
   }

   /**
    * Returns the parent SoapMethodNameMap
    *
    * @return parent SoapMethodNameMap
    */
   public SoapMethodNameMap getParent(  )
   {
      return m_parent;
   }

   /**
    * Returns a default name for a method based on a request's WS-Addressing Action header.
    * Subclasses that wish to support mapping based on wsa:Action should override this method.
    *
    * @param action string representation of WS-Addressing Action header value
    *
    * @return a method name, or null if no mapping is configured for the specified wsa:Action
    */
   protected String lookupMethodNameFromAction( String action )
   {
      return null;
   }
}