/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.resource.metadataexchange.v2004_09.porttype.impl;

import org.apache.ws.Soap1_1Constants;
import org.apache.ws.metadata.MetadataConfiguration;
import org.apache.ws.metadata.MetadataEntry;
import org.apache.ws.resource.AbstractPortType;
import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.ResourceContext;
import org.apache.ws.resource.faults.FaultException;
import org.apache.ws.resource.metadataexchange.v2004_09.MetadataExchangeConstants;
import org.apache.ws.resource.metadataexchange.v2004_09.porttype.MetadataExchangePortType;
import org.apache.ws.resource.properties.v2004_11.impl.NamespaceVersionHolderImpl;
import org.apache.ws.util.XmlBeanUtils;
import org.apache.xmlbeans.XmlObject;
import org.xmlsoap.schemas.ws.x2004.x08.addressing.EndpointReferenceType;
import org.xmlsoap.schemas.ws.x2004.x09.mex.AnyXmlType;
import org.xmlsoap.schemas.ws.x2004.x09.mex.GetMetadataDocument;
import org.xmlsoap.schemas.ws.x2004.x09.mex.LocationDocument;
import org.xmlsoap.schemas.ws.x2004.x09.mex.MetadataDocument;
import org.xmlsoap.schemas.ws.x2004.x09.mex.MetadataReferenceDocument;
import org.xmlsoap.schemas.ws.x2004.x09.mex.MetadataSectionDocument;
import java.net.URI;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * An implementation of the WS-MEX MetadataExchange portType.
 *
 * @author Sal Campana
 */
public class MetadataExchangePortTypeImpl
   extends AbstractPortType
   implements MetadataExchangePortType
{
   private static final NamespaceVersionHolder NAMESPACE_SET = new NamespaceVersionHolderImpl(  );

   /**
    * Creates a new {@link AbstractPortType} object.
    *
    * @param resourceContext
    */
   public MetadataExchangePortTypeImpl( ResourceContext resourceContext )
   {
      super( resourceContext );
   }

   /**
    * WS-MetadataExchange GetMetadata operation.
    *
    * @param request
    *
    * @return MetadataDocument response document
    */
   public MetadataDocument getMetadata( GetMetadataDocument request )
   {
      MetadataConfiguration metadataConfig = getResourceContext(  ).getResourceHome(  ).getMetadataConfig(  );

      // prep the response
      MetadataDocument          responseDoc      = createMetadataResponseDoc(  );
      MetadataDocument.Metadata responseMetadata = responseDoc.addNewMetadata(  );

      if ( metadataConfig != null )
      {
         // get the values from the request
         GetMetadataDocument.GetMetadata requestMetadata = request.getGetMetadata(  );
         String                          dialect    = requestMetadata.getDialect(  );
         String                          identifier = requestMetadata.getIdentifier(  );

         // get the metatdata from the config
         // the map is keyed on dialect and keyed to a List of metadata
         // this allows you to build a metadata section per dialect
         Map metadataMap = metadataConfig.getMetadata( dialect, identifier );

         // iterate the returned set and add metadata sections
         Iterator iterator = metadataMap.keySet(  ).iterator(  );
         while ( iterator.hasNext(  ) )
         {
            String dialectKey = (String) iterator.next(  );

            // get the list of metatadata for this dialect
            List metadataList = (List) metadataMap.get( dialectKey );

            for ( int i = 0; i < metadataList.size(  ); i++ )
            {
               // create a new metadata section
               MetadataSectionDocument.MetadataSection metadataSection =
                  responseMetadata.addNewMetadataSection(  );
               MetadataEntry                           metadataEntry = (MetadataEntry) metadataList.get( i );
               XmlObject                               metadata      = metadataEntry.getMetadata(  );
               if ( metadata == null )
               {
                  continue;
               }

               metadataSection.setDialect( dialectKey );
               if ( ( identifier != null ) && !"".equals( identifier ) )
               {
                  metadataSection.setIdentifier( identifier );
               }

               /** Determine type of metadata to add * */
               if ( metadata instanceof MetadataReferenceDocument )
               {
                  // adds a MetadataReference for locating the metadata
                  metadataSection.setMetadataReference( (EndpointReferenceType) metadata );
               }
               else if ( metadata instanceof LocationDocument )
               {
                  // adds a Location for locating the metadata
                  metadataSection.setLocation( ( (LocationDocument) metadata ).getLocation(  ) );
               }
               else
               {
                  // else its the metadata document and must be added as a
                  // child for the "any"
                  XmlBeanUtils.addChildElement( metadataSection, (XmlObject) metadata );
               }
            }
         }
      }

      getResourceContext(  ).setResponseAction( URI.create( MetadataExchangeConstants.ACTION_GETMETADATA_RESPONSE ) );
      return responseDoc;
   }

   /**
    * WS-MetadataExchange Get operation.
    *
    * @return AnyXmlType response document.
    */
   public AnyXmlType get(  )
   {
      //todo spec says may use WS-Addressing ActionNotSupported
      throw new FaultException( Soap1_1Constants.FAULT_SERVER, "Unimplemented Operation." );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   protected NamespaceVersionHolder getNamespaceSet(  )
   {
      return NAMESPACE_SET;
   }

   /**
    * Creates the response object for a GetMetadata request
    *
    * @return empty MetadataDocument
    */
   private MetadataDocument createMetadataResponseDoc(  )
   {
      return MetadataDocument.Factory.newInstance(  );
   }
}