/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.util.i18n.Keys;
import org.apache.ws.util.i18n.Messages;
import org.apache.ws.util.i18n.MessagesImpl;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

/**
 * Generic I/O utility methods.
 *
 * @author Ian Springer, Sal Campana
 */
public abstract class IoUtils
{
   private static final Log      LOG = LogFactory.getLog( IoUtils.class );
   private static final Messages MSG = MessagesImpl.getInstance(  );

   /**
    * Copies an input stream to an output stream.
    *
    * @param in  an input stream; must not be null
    * @param out an output stream; must not be null
    *
    * @throws IOException if the copy fails
    */
   public static void copy( InputStream  in,
                            OutputStream out )
   throws IOException
   {
      if ( in == null )
      {
         throw new IllegalArgumentException( MSG.getMessage( Keys.INPUTSTRM_MST_NOT_B_NULL ) );
      }

      if ( out == null )
      {
         throw new IllegalArgumentException( MSG.getMessage( Keys.OUTSTRM_MST_NOT_B_NULL ) );
      }

      // use Buffered streams for better performance
      in     = new BufferedInputStream( in );
      out    = new BufferedOutputStream( out );

      // do not allow other threads to read from the input or write to
      // the output while copying is taking place
      synchronized ( in )
      {
         synchronized ( out )
         {
            byte[] bytes     = new byte[8192];
            int    bytesRead;
            while ( ( bytesRead = in.read( bytes ) ) != -1 )
            {
               out.write( bytes, 0, bytesRead );
            }

            out.flush(  );
         }
      }
   }

   /**
    * Converts a ByteArrayOutputStream into a ByteArrayInputStream.
    *
    * @param out
    *
    * @return ByteArrayInputStream
    */
   public static ByteArrayInputStream toByteArrayInputStream( ByteArrayOutputStream out )
   {
      return new ByteArrayInputStream( out.toByteArray(  ) );
   }

   /**
    * Convert the specified string to an input stream.
    *
    * @param str a string
    *
    * @return an input stream
    */
   public static InputStream toInputStream( String str )
   {
      LOG.debug( MSG.getMessage( Keys.ATTMPT_CNVRT_STRING_TO_INPTSTRM, str ) );
      try
      {
         return ( new ByteArrayInputStream( str.getBytes( "UTF-8" ) ) );
      }
      catch ( UnsupportedEncodingException uee )
      { // this should never occur - UTF-8 is always supported
         uee.printStackTrace(  );

         // return using the current locale encoding
         return ( new ByteArrayInputStream( str.getBytes(  ) ) );
      }
   }
}